import click
from click.core import Context

CMD_ALIAS_MAP = {
    "ls": "list",
}


class AliasedGroup(click.Group):
    def get_command(self, ctx: Context, cmd_name: str):
        if cmd_name in CMD_ALIAS_MAP:
            cmd_name = CMD_ALIAS_MAP[cmd_name]
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx) if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail(f"Too many matches: {', '.join(sorted(matches))}")

    def resolve_command(self, ctx: Context, args: list[str]):
        # always return the full command name
        _, cmd, args = super().resolve_command(ctx, args)
        return cmd.name, cmd, args



import importlib
import logging
import os.path
import pathlib
import sys
from importlib.machinery import SourceFileLoader
from types import ModuleType

import click
import rich
from rich import box
from rich.markup import escape
from rich.padding import Padding
from rich.table import Table

from ...artifacts.registry import collect
from ...artifacts.registry import Registry
from ..environment import Environment
from ..environment import pass_env
from .cli import cli

logger = logging.getLogger(__name__)
TABLE_HEADER_STYLE = "yellow"
TABLE_COLUMN_STYLE = "cyan"


def load_module(module_spec: str) -> ModuleType:
    if module_spec.lower().endswith(".py") and os.path.exists(module_spec):
        module_path = pathlib.Path(module_spec)
        module_name = module_path.stem
        return SourceFileLoader(module_name, str(module_path)).load_module(module_name)
    try:
        current_dir = str(pathlib.Path.cwd())
        sys.path.insert(0, current_dir)
        return importlib.import_module(module_spec)
    finally:
        del sys.path[0]


def collect_artifacts(module_spec: str) -> Registry:
    registry = collect([load_module(module_spec)])
    if not registry.artifacts:
        logger.error("No artifacts found")
    return registry


@cli.command(name="list", help="List artifacts")
@click.argument("MODULE")
@pass_env
def list_artifacts(env: Environment, module: str):
    registry = collect_artifacts(module)

    env.logger.info(
        "Listing artifacts for [blue]%s[/]",
        module,
        extra={"markup": True, "highlighter": None},
    )

    table = Table(
        title="Artifacts",
        box=box.SIMPLE,
        header_style=TABLE_HEADER_STYLE,
        expand=True,
    )
    table.add_column("Module", style=TABLE_COLUMN_STYLE)
    table.add_column("Name", style=TABLE_COLUMN_STYLE)
    table.add_column("Sample", style=TABLE_COLUMN_STYLE)
    for module, artifacts in registry.artifacts.items():
        for i, (name, artifact) in enumerate(artifacts.items()):
            table.add_row(
                escape(module) if i == 0 else "", escape(name), str(artifact.sample)
            )
    rich.print(Padding(table, (1, 0, 0, 4)))


@cli.command(help="View artifact")
@click.argument("MODULE")
@click.argument("ARTIFACTS", nargs=-1)
@click.option(
    "-p", "--port", help="OCP Viewer port to send the model data to", default=3939
)
@pass_env
def view(env: Environment, module: str, artifacts: tuple[str, ...], port: int):
    registry = collect_artifacts(module)
    if not artifacts:
        target_artifact = list(list(registry.artifacts.values())[0].values())[0]
        logger.info(
            "No artifacts provided, use the first one %s/%s",
            target_artifact.module,
            target_artifact.name,
        )
        target_artifacts = [target_artifact]
    else:
        if len(registry.artifacts) > 1:
            raise ValueError("Unexpected more than one modules found")
        module_artifacts = list(registry.artifacts.values())[0]
        target_artifacts = [
            module_artifacts[artifact_name] for artifact_name in artifacts
        ]

    # TODO: this is going to be a bit slow, provide a progress bar & cache?
    realized_artifacts = [artifact.func() for artifact in target_artifacts]
    # defer the import to make testing mocking much easier
    from ocp_vscode import show

    # TODO: pass in some args
    show(realized_artifacts, port=port)


@cli.command(help="Export artifact")
@click.argument("MODULE")
@pass_env
def export(env: Environment, module: str):
    registry = collect_artifacts(module)
    # TODO:



import typing

import venusian

from .. import constants
from .data_types import Artifact


def artifact(
    func: typing.Callable | None = None, *, sample: bool = False
) -> typing.Callable:
    def decorator(wrapped: typing.Callable):
        artifact_obj = Artifact(
            module=wrapped.__module__,
            name=wrapped.__name__,
            func=wrapped,
            sample=sample,
        )

        def callback(scanner: venusian.Scanner, name: str, ob: typing.Callable):
            if artifact_obj.name != name:
                raise ValueError("Name is not the same")
            scanner.registry.add(artifact_obj)

        venusian.attach(
            wrapped,
            callback,
            category=constants.MR_ARTIFACTS_CATEGORY,
            depth=1 if func is None else 2,
        )
        return wrapped

    if func is not None:
        return decorator(func)

    return decorator



import asyncio
import json
import pathlib

import pytest
import pytest_asyncio
import websockets
from click.testing import CliRunner
from pytest import MonkeyPatch

from .helper import switch_cwd
from mr.cmds.main import cli


DEFAULT_MOCK_CONFIG = {
    "ambient_intensity": 1.0,
    "angular_tolerance": 0.2,
    "axes": False,
    "axes0": False,
    "black_edges": False,
    "center_grid": False,
    "collapse": "R",
    "control": "trackball",
    "debug": False,
    "default_color": "#e8b024",
    "default_edgecolor": "#808080",
    "default_facecolor": "Violet",
    "default_opacity": 0.5,
    "default_thickedgecolor": "MediumOrchid",
    "default_vertexcolor": "MediumOrchid",
    "deviation": 0.1,
    "direct_intensity": 1.1,
    "explode": False,
    "grid": [False, False, False],
    "grid_font_size": 12,
    "metalness": 0.3,
    "modifier_keys": {
        "shift": "shiftKey",
        "ctrl": "ctrlKey",
        "meta": "metaKey",
        "alt": "altKey",
    },
    "new_tree_behavior": True,
    "glass": True,
    "tools": True,
    "pan_speed": 0.5,
    "ortho": True,
    "reset_camera": "RESET",
    "rotate_speed": 1.0,
    "roughness": 0.65,
    "theme": "browser",
    "ticks": 5,
    "transparent": False,
    "tree_width": 240,
    "up": "Z",
    "zoom_speed": 0.5,
    "_splash": True,
}
DEFAULT_MOCK_STATUS = {
    "ambient_intensity": 1,
    "states": {"/Group/Solid": [1, 1]},
    "direct_intensity": 1.1,
    "clip_slider_0": 40,
    "clip_slider_1": 40,
    "clip_slider_2": 40,
    "tab": "tree",
    "target": [10.4019, 0.4274, -24.7534],
    "target0": [10.4019, 0.4274, -24.7534],
    "position": [101.36996653681135, -93.26906574513457, -1.8075756537109555],
    "quaternion": [0.5963, 0.2408, 0.2863, 0.7099],
    "zoom": 0.9999999999999998,
}


class MockMsgHandler:
    def __init__(self, config: dict | None = None, status: dict | None = None):
        self.config = config or DEFAULT_MOCK_CONFIG
        self.status = status or DEFAULT_MOCK_STATUS
        self.data_msgs = []
        self.backend_msgs = []

    async def __call__(self, websocket: websockets.WebSocketServerProtocol, path: str):
        async for data in websocket:
            msg_type = data[:1]
            payload = data[2:]
            match msg_type:
                # command
                case b"C":
                    cmd = json.loads(payload)
                    match cmd:
                        case "status":
                            await websocket.send(json.dumps(self.status))
                        case "config":
                            await websocket.send(json.dumps(self.config))
                case b"D":
                    self.data_msgs.append(payload)
                case b"B":
                    self.backend_msgs.append(payload)


@pytest.fixture
def msg_handler() -> MockMsgHandler:
    return MockMsgHandler()


@pytest_asyncio.fixture
async def ws_server(msg_handler: MockMsgHandler, unused_tcp_port: int):
    server = await websockets.serve(msg_handler, "localhost", unused_tcp_port)
    try:
        yield server
    finally:
        server.close()
        await server.wait_closed()


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "module",
    [
        "examples/main.py",
        "examples.main",
    ],
)
async def test_view(
    monkeypatch: MonkeyPatch,
    cli_runner: CliRunner,
    fixtures_folder: pathlib.Path,
    module: str,
    unused_tcp_port: int,
    ws_server: websockets.WebSocketServer,
    msg_handler: MockMsgHandler,
):
    monkeypatch.syspath_prepend(fixtures_folder)

    def operate():
        with switch_cwd(fixtures_folder):
            result = cli_runner.invoke(
                cli,
                ["artifacts", "view", module, "main", "-p", unused_tcp_port],
                catch_exceptions=False,
            )
        assert result.exit_code == 0

    await asyncio.wait_for(asyncio.to_thread(operate), 10)
    assert len(msg_handler.data_msgs) == 1
    assert len(msg_handler.backend_msgs) == 1



import sys

from mr import Artifact
from mr import artifact
from mr.artifacts.registry import collect


@artifact
def artifact_without_params():
    return "artifact_without_params"


@artifact(sample=True)
def sample_artifact():
    return "sample_artifact"


def test_collect():
    module = sys.modules[__name__]
    registry = collect([module])
    assert registry.artifacts == {
        __name__: {
            artifact_without_params.__name__: Artifact(
                module=__name__,
                name=artifact_without_params.__name__,
                func=artifact_without_params,
                sample=False,
            ),
            sample_artifact.__name__: Artifact(
                module=__name__,
                name=sample_artifact.__name__,
                func=sample_artifact,
                sample=True,
            ),
        }
    }
    assert artifact_without_params() == "artifact_without_params"
    assert sample_artifact() == "sample_artifact"
